import plotly
from IPython.core.display import display, HTML
from expai.utils import generate_response

class ExpaiExplanation:
    """
    Class to wrap a explanation
    """
    def __init__(self, model_id: str,
                 project_id: str,
                 sample_id: str,
                 access_token: str,
                 headers: dict,
                 server_name: str,
                 session,
                 project,
                 visualizations,
                 explanation_type_id,
                 json_details):

        self.model_id = model_id
        self.project_id = project_id
        self.sample_id = sample_id

        self.server_name = server_name
        self.access_token = access_token

        self.headers = headers

        self.sess = session
        self.project = project

        self.json_details = json_details
        self.explanation_type_id = explanation_type_id
        self.visualizations = visualizations

    def get_keys(self):
        """
        Returns the keys of every single plot available in the object.
        """
        return list(self.visualizations.keys())

    def plot(self, key: str = None):
        """
        Plot a single visualization from the object
        :param key: key of the visualization to be plotted
        """
        assert key is not None, "You must specify a key to be plotted. You can use .get_keys() to get all available keys."
        assert key in self.visualizations.keys(), "Your key doesn't exist in this explanation. You can use .get_keys() to get all available keys."

        try:
            display(HTML(plotly.io.to_html(plotly.io.from_json(str(self.visualizations[key])))))
        except:
            raise Exception("There was an error creating the plot. Please, try again.")

    def plot_all(self):
        """
        Plots all visualizations stored in the object one after the other.
        """
        def figures_to_html(figs):
            # Create unique HTML for all figures
            html = "<html><head></head><body>" + "\n"
            for fig in figs:
                inner_html = plotly.io.from_json(str(fig)).to_html().split('<body>')[1].split('</body>')[0]
                html += inner_html
            html += "</body></html>\n"
            return html

        if isinstance(self.visualizations, dict):
            html = figures_to_html(list(self.visualizations.values()))

        elif isinstance(self.visualizations, list):
            html = figures_to_html(self.visualizations)

        else:
            raise Exception("The input type for plots must be dictionary or list containing only explanations.")

        display(HTML(html))


    def store_explanation(self,
                          explanation_name: str = None,
                          explanation_summary: str = None,
                          keys: list = None):
        """
        Store the explanation in your project for later reuse.
        :param explanation_name: Name to identify the explanation
        :param explanation_summary: Description for the plot.
        """
        assert explanation_name is not None, "You must specify a name for the explanation."
        if keys:
            assert set(keys).issubset(self.visualizations.keys()), "Some of the keys you introduce are not available. Please, try again."
        else:
            keys = self.visualizations.keys()

        data = self.json_details
        data['explanation_type_id'] = self.explanation_type_id
        data['explanation_name_des'] = explanation_name
        data['explanation_summary_des'] = explanation_summary
        data['explanation_json'] = {k: v for k, v in self.visualizations.items() if k in keys}
        data['model_id'] = self.model_id
        data['sample_id'] = self.sample_id

        response = self.sess.request("POST", self.server_name + "/api/projects/{}/explanation/create".format(self.project_id),
                                    json=data, headers=self.headers)

        if response.ok:
            return "Explanation successfully created with id {}".format(response.json()['explanation_id'])
        else:
            return generate_response(response)






import logging
from IPython.core.display import display, HTML
from expai.utils import generate_response
import numpy as np


class ExpaiModelFairness:
    def __init__(self, model_id: str, project_id: str, access_token: str, headers: dict, server_name: str, session, project):
        self.model_id = model_id
        self.project_id = project_id

        self.server_name = server_name
        self.access_token = access_token

        self.headers = headers

        self.sess = session
        self.project = project

    def launch_fairness_dashboard(self, sample_name: str = None, sample_id: str = None, subset_indexes: list = None, protected_columns: list = None):
        assert sample_name is not None or sample_id is not None, "You must provide a sample name or id for the explanation"
        if subset_indexes is None:
            logging.warning(
                "You will generate explanation for all entries in the file. This might consume many credits and time. Use it on your own risk.")

        if sample_id is None:
            sample_id = self._get_sample_id_from_name(sample_name)

            if sample_id is None:
                logging.error(
                    "We could not find any sample matching that name. Please, try again or use sample_id as parameter")
                return None

        json = {
            "sample_id": sample_id,
            "subset_indexes": subset_indexes,
            "protected_columns": ",".join(protected_columns) if protected_columns else None
        }

        response = self.sess.request("POST",
                                     self.server_name + "/api/fairness/{}/dashboard".format(self.model_id),
                                     headers=self.headers, json=json)

        if response.ok:
            html = response.json()['html']
            display(HTML(html), metadata=dict(isolated=True))
            return html

        else:
            return generate_response(response, ['html'])


    def _get_sample_id_from_name(self, sample_name: str = None):
        sample_list = self.project.sample_list()
        sample_list = sample_list.get('samples')

        if sample_list is None:
            return None

        for sample in sample_list:
            if sample['sample_name_des'] == sample_name:
                return sample['sample_id']
        return None



import requests
import logging
from expai.expai_project import ExpaiProject
from expai.utils import generate_response


class ExpaiLogin:
    def __init__(self, email: str = None, user_pass: str = None, access_token: str = None, server_name: str = "https://api.expai.io:443"):
        assert email is not None and user_pass is not None or access_token is not None, "Please, introduce your email and password or access_token"

        self.server_name = server_name
        self.email = email

        if access_token is None:
            self.headers = {
                'Content-Type': 'application/json'
            }

            json = {
                "email_des": email,
                "password_des": user_pass
            }

            self.sess = requests.session()
            response = self.sess.post(self.server_name + "/api/auth/login", headers=self.headers, json=json)

            if not response.ok:
                logging.error("Invalid credentials, please check them and try again. Thank you.")

            self.access_token = response.json().get('access_token')
            self.headers['access_token'] = self.access_token

        else:
            self.access_token = access_token

            self.headers = {
                'Content-Type': 'application/json',
                'access_token': access_token
            }

            self.sess = requests.session()
            response = self.sess.get(self.server_name + "/api/auth/check", headers=self.headers)

    def project_list(self, search_by: str = None, exact_search: str = None):
        """
        Returns the list of projects for the user
        :return: list of projects
        """

        if exact_search is not None:
            response = self.sess.request("GET",
                                         self.server_name + "/api/projects/list/contains/{}".format(exact_search),
                                         headers=self.headers)

            if 'projects' in response.json().keys():
                for project in response.json()['projects']:
                    if project['project_name_des'] == exact_search:
                        return project
            logging.error(
                "We could not find a sample with this exact name. Try again or consider using search_by parameter to find similar samples")
            return None

        elif search_by is not None:
            response = self.sess.request("GET",
                                         self.server_name + "/api/projects/list/contains/{}".format(search_by),
                                         headers=self.headers)
            return generate_response(response, ['projects'])

        else:
            response = self.sess.request("GET", self.server_name + "/api/projects/list", headers=self.headers)

            return generate_response(response, ['projects'])


    def create_project(self, project_name: str = None, project_description: str = None):
        """
        Create a new project
        :param project_id: Name for the project
        :param project_description: Description for the project
        :return: True if successful
        """
        assert project_name is not None, "Please, introduce a project_id"

        payload = {"project_name_des": project_name,
                   'project_description_des': project_description}

        response = self.sess.request("POST", self.server_name + "/api/projects/create", headers=self.headers,
                                     json=payload)

        if response.ok:
            return "Project successfully created with id: {}".format(response.json()['project_id'])
        else:
            return generate_response(response)

    def delete_project(self, project_name: str = None, project_id: str = None):
        """
        Delete a project. You must indicate project_id or project_name
        :param project_id: id for the project (optional).
        :param project_name: name of the project.
        :return:
        """
        assert project_id is not None or project_name is not None, "To delete a project you must specify its project_id or project_name"

        if project_id is None:
            project_id = self._get_project_id_from_name(project_name)

        response = self.sess.request("DELETE", self.server_name + "/api/projects/{}".format(project_id),
                                     headers=self.headers)

        if response.ok:
            return "Project successfully deleted"
        else:
            return generate_response(response)

    def get_project(self, project_name: str = None, project_id: str = None):
        """
        Get a Python object that will enable you to interact with a project and its models.
        :param project_name: Name of the project
        :param project_id: Id of the project (optional)
        :return: ExpaiProject object
        """
        assert project_name is not None or project_id is not None, "You must provide a project name or project id"

        if project_id is None:
            project_id = self._get_project_id_from_name(project_name)

        if project_id is not None:
            return ExpaiProject(project_id, self.access_token, self.headers, self.server_name, self.sess)
        else:
            logging.error("We could not find your project. Please, check your name or try using project_id as parameter")
            return None

    def update_project(self, project_name: str = None, project_id: str = None, update_info: dict = None):
        """
        Update a project
        """
        assert project_id is not None or project_name is not None, "You must indicate the project_name or project_id"
        assert update_info is not None, "You must provide the update_info dictionary with the values to be updated"

        if project_id is None:
            project_id = self._get_project_id_from_name(project_name)

            if project_id is None:
                logging.error(
                    "We could not find any sample matching that name. Please, try again or use sample_id as parameter")
                return False

        response = self.sess.request("PATCH",
                                    self.server_name + "/api/projects/{}".format(project_id),
                                    headers=self.headers, json=update_info)

        if response.ok:
            return "Project with id {} successfully updated".format(project_id)
        else:
            return generate_response(response)

    def _get_project_id_from_name(self, project_name: str = None):
        project = self.project_list(exact_search=project_name)
        if project is not None:
            return project['project_id']

        return None

    def get_allowed_models(self):
        """
        Get a Python object that will enable you to interact with a project and its models.
        """
        response = self.sess.request("GET",
                                     self.server_name + "/api/models/get_allowed",
                                     headers=self.headers)

        return generate_response(response, ['models_category', 'models_objective', 'models_prediction', 'models_library'])

    def get_allowed_files(self):
        """
        Get a Python object that will enable you to interact with a project and its models.
        """
        response = self.sess.request("GET",
                                     self.server_name + "/api/files/get_allowed",
                                     headers=self.headers)

        return generate_response(response, ["files_type"])



explanation_types = {'Explicación del modelo': 1,
                     'Explicación de la variable': 2,
                     'Explicación de la muestra': 3,
                     'What If': 4,
                     'What If Battle': 5}


from expai.expai_requests import ExpaiLogin
import os

if __name__ == '__main__':
    expaireq = ExpaiLogin(email="test@test.com", user_pass="test", access_token="6H19l-zXUzJVli2bcEMENssBjx0Ph4dnI0rmNtWcVac")
    models = expaireq.get_allowed_models()
    projects = expaireq.project_list(search_by='Proy')
    project = expaireq.get_project(project_name='Proyecto Test DAG')
    explainer_xgb = project.get_model_explainer("Car Selling Price Predictor XGB")
    project.create_sample(sample_path=os.path.abspath("./car_ad_display.csv"),
                          sample_name="Test Display",
                          sample_separator=";",
                          sample_encoding="latin-1",
                          is_display=True,
                          reference_sample_name="Test")
    test = project.create_sample(sample_path='/Users/javi/Desktop/subir1.csv',
                          sample_name="Sample prueba lista5", sample_separator=",", is_display=False)
    project.append_sample(sample_path='/Users/javi/Desktop/subir2.csv',
                          sample_name="Sample prueba lista5")

    probando = project.get_sample(sample_name="Sample prueba lista5")
    projects = expaireq.project_list(exact_search='Proyecto 1')
    projects = expaireq.project_list(exact_search='Proyec 1')
    response = expaireq.delete_project(project_name="Prueba pycharm 2")
    #response = expaireq.create_project("Prueba pycharm 2")
    project = expaireq.get_project(project_name='Explanation test')
    project.create_sample(sample_path='/Users/javi/Desktop/Trabajo/Expai/expai/test/propension/X.csv',
                          sample_name="Sample 1", sample_separator=",", sample_target_col='y')
    project.create_model("/Users/javi/Desktop/Trabajo/Expai/expai/test/propension/model.pkl", True, "Model 1")
    explainer = project.get_model_explainer(model_name="Test model")
    shap_values = explainer.raw_explanation(sample_name="primera prueba con target aaaa", indexes=[0, 1])
    print(response)




from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()

setup_args = dict(
    name='expai',
    version='1.3.7.4',
    description='User-friendly library to interact with EXPAI',
    long_description_content_type="text/markdown",
    long_description=README + '\n\n' + HISTORY,
    license='MIT',
    packages=find_packages(),
    author='EXPAI',
    author_email='info@expai.io',
    keywords=['EXPAI', 'ExplainableAI', 'Explainable Artificial Intelligence'],
    url='https://expai.io',
    download_url='https://pypi.org/project/expai/'
)

install_requires = ['requests', 'ipython', 'pandas', 'numpy', 'plotly']

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires)